from . import miscfuncs
    
def gr_clay_shale_vol(minvalue, maxvalue, inputvalue, method="linear", limit_result=False, low_limit=0, high_limit=1):
    """
    Calculates a clay or shale volume from gamma ray log data.
    
    Maxvalue can be set to represent 100% shale value when working with VShale, or it can
    be set to 100% clay when working with VClay.

    Non-linear equations can be selected using the method argument:
        larionov-young
        larionov-old
        steiber
        clavier

    Parameters
    ----------
    minvalue : float
        Value representing a 100% clean interval.
    maxvalue : float
        Value representing either 100% clay or 100% shale.
    inputvalue : float
        Gamma ray value from log measurements.
    method : string
        Select method for calculating VClay or VShale:
            linear
            larionov-young
            larionov-old
            steiber
            clavier
        By default: linear
    limit_result : bool, optional
        Apply limits to the result value.
        By default False
    low_limit : int, optional
        Low limit. If value falls below this limit it will be set to this value. 
        By default 0
    high_limit : float, optional
        High limit. If value falls above this limit it will be set to this value.
        By default: 1

    Returns
    -------
    float
        Returns a VShale or VClay in decimal units.
    
    References
    -------
    Larionov VV (1969) Borehole radiometry: Moscow, U.S.S.R., Nedra
    Steiber RG (1973) Optimization of shale volumes in open hole logs. J Petrol Technol 1973(31):147–162

    Asquith and Krygowski (2004) Basic Well Log Analysis. Second Edition.
    Original paper reference cannot be found for Clavier (1971), however, the following discussion provides insight to its origins.
    https://www.researchgate.net/post/Volume_fraction_of_shale_full_reference_for_Clavier_1971
    """
    
    igr = (inputvalue - minvalue)/(maxvalue - minvalue)

    if method == "linear":
        result = igr
    elif method == "larionov-young":
        result = 0.083 * ((2**(3.71 * igr)) - 1)
    elif method == "larionov-old":
        result = 0.33 * ((2**(2 * igr)) - 1)
    elif method == "steiber":
        result = igr / (3 - 2 * igr)
    elif method == "clavier":
        result = 1.7 - ((3.38-(igr + 0.7)**2)**0.5)
    else:
        print("Enter a valid method string.")

    if limit_result is True:
        return miscfuncs.limit_vals(result, low_limit, high_limit)
    else:
        return result


from setuptools import setup

setup(
    name='pypetrophysics',
    url='https://github.com/andymcdgeo/PyPetrophysics',
    author='Andy McDonald',
    email='andymcdonaldpetro@gmail.com',
    packages=['pypetrophysics'],
    version='0.1.2',
    description='A library of petrophysical calculations',
    long_description=open('README.md').read()

)


def limit_vals(input_value, low_limit, high_limit):
    """
    Apply limits to an input value.

    Parameters
    ----------
    input_value : float
        Input value.
    low_limit : float
        Low limit. If value falls below this limit it will be set to this value.
    high_limit : float
        High limit. If value falls above this limit it will be set to this value.

    Returns
    -------
    float
        Returns input value unless it falls above or below the entered limits.
    """
    if input_value < low_limit:
        return low_limit
    elif input_value > high_limit:
        return high_limit
    else:
        return input_value

def dec_perc_convert(input_value, input_units):
    """
    Convert from decimal to percent or percent to decimal.

    Parameters
    ----------
    input_value : float
        Value to be converted.
    input_units : string
        Units of the input value.
        Enter either "percent" or "decimal"

    Returns
    -------
    float
        Returns converted value in percent or decimal.
    """
    if input_units == "percent":
        return input_value / 100
    elif input_units == "decimal":
        return input_value * 100
    else:
        print("Enter the correct value for input units: percent or decimal")



class Porosity:

    def limit_vals(self, input_value, low_limit, high_limit):
        """
        Apply limits to an input value.

        Parameters
        ----------
        input_value : float
            Input value.
        low_limit : float
            Low limit. If value falls below this limit it will be set to this value.
        high_limit : float
            High limit. If value falls above this limit it will be set to this value.

        Returns
        -------
        float
            Returns input value unless it falls above or below the entered limits.
        """
        if input_value < low_limit:
            return low_limit
        elif input_value > high_limit:
            return high_limit
        else:
            return input_value

    def porosity_density(self, rhomatrix, rhofluid, rhobulk, limit_result=False, low_limit=0, high_limit=0.6):
        """
        Calculate porosity from a density log

        Parameters
        ----------
        rhomatrix : float
            Matrix density. 
            
            Typical values:
              Sandstone: 2.65 g/cc
              Limestone: 2.71 g/cc
              Dolomite: 2.80 - 2.85 g/cc
        rhofluid : float
            Fluid density.
        rhobulk : float
            Bulk density from log measurements
        limit_result : bool, optional
            Apply limits to the result value.
            By default False
        low_limit : int, optional
            Low limit. If value falls below this limit it will be set to this value. 
            By default 0
        high_limit : float, optional
            High limit. If value falls above this limit it will be set to this value.
            By default: 0.6

        Returns
        -------
         float
            Density porosity value in decimal units.
        """
        porosity = (rhomatrix - rhobulk)/(rhobulk - rhofluid)

        if limit_result is True:
            return self.limit_vals(porosity, low_limit, high_limit)
        else:
            return porosity
            
    def porosity_sonic(self, dtmatrix, dtfluid, dtlog, method="wylie", limit_result=False, low_limit=0, high_limit=0.6):
        """
        Caculate porosity from a sonic log using either the Wylie-Time Average equation
        or Raymer-Hunt-Gardener equation.

        Parameters
        ----------
        dtmatrix : float
            Matrix slowness.

            Typical values:
              Sandstone: 52-55 us/ft
              Limestone: 47 us/ft
              Dolomite: 43 us/ft
        dtfluid : float
            Fluid slowness.
        dtlog : [type]
            Slowness (DT) from log measurements.
        method : string
            Select a method for calculating sonic porosity:
            "wylie" - Wylie Time Average (default)
            "raymer" - Raymer Hunt Gardener
        limit_result : bool, optional
            Apply limits to the result value.
            By default False
        low_limit : int, optional
            Low limit. If value falls below this limit it will be set to this value. 
            By default 0
        high_limit : float, optional
            High limit. If value falls above this limit it will be set to this value.
            By default: 0.6

        Returns
        -------
        float
            Sonic porosity value in decimal units.

        Raises
        ------
        Exception
            Raise an exception if method value is not 1 or 2.

        References
        ----------
        Wyllie, M.R.J., Gregory, A.R., and Gardner, L.W. 1956. Elastic Wave Velocities in Heterogeneous and Porous Media. Geophysics 21 (1): 41–70.
        Raymer, L.L., Hunt, E.R., and Gardner, J.S. 1980. An Improved Sonic Transit Time-to-Porosity Transform, paper P. Trans., 1980 Annual Logging Symposium, SPWLA, 1–12.
        """
        if method == "wylie": #Wylie Time Average
            porosity = (dtlog - dtmatrix) / (dtfluid - dtmatrix)
        elif method == "raymer": #Raymer Hunt Gardener 
            alpha = (dtmatrix / (2 * dtfluid)) - 1
            porosity = -alpha-((alpha**2 + (dtmatrix / dtlog)-1)**0.5)
        else:
            raise Exception("Enter a valid method value: wylie or raymer")

        if limit_result is True:
            return self.limit_vals(porosity, low_limit, high_limit)
        else:
            return porosity
